//
//  UMLayer.h
//  ulib.framework
//
//  Created by Andreas Fink on 10.11.11.
//  Copyright (c) Andreas Fink (andreas@fink.org), Basel, Switzerland. All rights reserved

#import <pthread.h>

#import "UMObject.h"
#import "UMLogLevel.h"

@class UMThroughputCounter;
@class UMTask;
@class UMTaskQueue;
@class UMTaskQueueMulti;
@class UMLayerTask;
@class UMLayerAckRequest;

#define UMLAYER_ADMIN_QUEUE             0
#define UMLAYER_LOWER_PRIORITY_QUEUE    1
#define UMLAYER_UPPER_PRIORITY_QUEUE    2
#define UMLAYER_LOWER_QUEUE             3
#define UMLAYER_UPPER_QUEUE             4
#define UMLAYER_QUEUE_COUNT             5


@interface UMLayer : UMObject
{
    BOOL                                sharedQueue;
    UMTaskQueueMulti                    *taskQueue;
    UMThroughputCounter					*lowerQueueThroughput;
    UMThroughputCounter					*upperQueueThroughput;
    UMThroughputCounter					*adminQueueThroughput;
    NSString                            *name;
    BOOL                                enable;
    NSString                            *logFileName;
    UMLogLevel                          logLevel;
}


- (UMLayer *)initWithTaskQueueMulti:(UMTaskQueueMulti *)tq;
- (NSString *)type;

@property(readwrite,strong) UMTaskQueueMulti *taskQueue;
@property(readwrite,assign) BOOL sharedQueue;


@property(readwrite,strong) UMThroughputCounter *lowerQueueThroughput;
@property(readwrite,strong) UMThroughputCounter *upperQueueThroughput;
@property(readwrite,strong) UMThroughputCounter *adminQueueThroughput;

//@property(readwrite,strong) UMLogFeed   *log;
@property(readwrite,strong) NSString    *name;
@property(readwrite,assign) BOOL        enable;


- (void)queueFromLower:(UMTask *)task;
- (void)queueFromUpper:(UMTask *)task;
- (void)queueFromAdmin:(UMTask *)task;
- (void)queueFromLowerWithPriority:(UMLayerTask *)task;
- (void)queueFromUpperWithPriority:(UMLayerTask *)task;



- (void)adminInit;
- (void)adminAttachFor:(UMLayer *)attachingLayer userId:(id)uid;
- (void)adminAttachConfirm:(UMLayer *)attachedLayer userId:(id)uid;
- (void)adminAttachFail:(UMLayer *)attachedLayer userId:(id)uid;

#pragma mark -
#pragma mark Layer Logging

- (void)logDebug:(NSString *)s;
- (void)logInfo:(NSString *)s;
- (void)logWarning:(NSString *)s;
- (void)logMinorError:(NSString *)s;
- (void)logMajorError:(NSString *)s;
- (void)logMajorError:(int)err location:(NSString *)location;
- (void)logMinorError:(int)err location:(NSString *)location;
- (void)logPanic:(NSString *)s;
- (void)addLayerConfig:(NSMutableDictionary *)config;
- (void)readLayerConfig:(NSDictionary *)config;
@end
