//
//  UniversalCoder.h
//  ulib
//
//  Created by Andreas Fink on 04.12.09.
//  Copyright (c) Andreas Fink (andreas@fink.org), Basel, Switzerland. All rights reserved
//

#ifndef UNIVERSALCODER_H
#define UNIVERSALCODER_H 1

#import <Foundation/Foundation.h>
#import "UMObject.h"
#import "UniversalCodingProtocol.h"
#import "UniversalCoderRegistry.h"
#import "UniversalCoderTypeCodes.h"

@interface UniversalCoder : NSObject
{
	size_t                      positionInInputStream;
	NSData                      *inputBuffer;
	NSMutableData               *outputBuffer;
	UniversalCoderRegistry		*typeRegistry;
	BOOL                        debug;
}

@property(readwrite,assign,nonatomic)	size_t positionInInputStream;
@property(readwrite,strong,nonatomic)   UniversalCoderRegistry *typeRegistry;
@property(readwrite,assign,nonatomic)   BOOL debug;

- (UniversalCoder *)init;
- (UniversalCoder *)initWithInputData:(NSData *)data;

- (NSData *)outputData;

- (void) appendBytes:(const unsigned char *)ptr length:(size_t)len withDebug:(BOOL)doDebug;
- (void) appendBytes:(const unsigned char *)ptr length:(size_t)len;
- (void) appendFlexibleInt:(int64_t)value withLength:(BOOL)includeLength;
- (void) appendTag:(M2Tag)tag;
- (void) appendType:(M2Type)type;
- (void) appendLength:(M2Length)length;
- (void) appendLengthAndString:(NSString *)string usingEncoding:(NSStringEncoding)encoding;
- (void) appendLengthAndUTF8String:(NSString *)string;
- (void) appendLengthAndData:(NSData *)data;

- (void) appendTagTypeLengthBoolean:(BOOL)b forTag:(M2Tag)tag;
- (void) appendTagTypeLengthDouble:(double)i	   forTag:(M2Tag)tag;
- (void) appendTagTypeLengthInteger:(int64_t)i	   forTag:(M2Tag)tag;
- (void) appendTagTypeLengthString:(NSString *)s   forTag:(M2Tag)tag;
- (void) appendTagTypeLengthData:(NSData *)d	   forTag:(M2Tag)tag;
- (void) appendTagTypeLengthArray:(NSArray *)array forTag:(M2Tag)tag;
- (void) appendTagTypeLengthDictionary:(NSDictionary *)dict  forTag:(M2Tag)tag;

- (void) appendTypeLengthDouble:(double)i;
- (void) appendTypeLengthInteger:(int64_t)i;
- (void) appendTypeLengthString:(NSString *)s;
- (void) appendTypeLengthData:(NSData *)d;
- (void) appendTypeLengthArray:(NSArray *)array;
- (void) appendTypeLengthDictionary:(NSDictionary *)dict;
- (void) appendTypeLengthBoolean:(BOOL)b;

- (void) appendObject:(id)item;

- (uint8_t)grabByte;
- (int64_t)grabFlexibleInt;

- (void) skipTypeLengthItem;
- (void) skipLengthItem;

- (M2Tag)      grabTag;
- (M2Type)     grabType;
- (M2Type)     peekType;
- (M2Length)   grabLength;

- (BOOL)	   grabTypeLengthBoolean;
- (int64_t)    grabTypeLengthInteger;
- (double)     grabTypeLengthDouble;
- (NSString *) grabTypeLengthString;
- (NSData *)   grabTypeLengthData;
- (NSArray *)  grabTypeLengthArray;
- (NSMutableArray *) grabTypeLengthMutableArray;
- (NSData *)   grabLengthData;
- (id) grabTypeLengthObject;

- (NSDictionary *) grabTypeLengthDictionary;
- (NSMutableDictionary *) grabTypeLengthMutableDictionary;


/* creates a new coder but with empty data buffer */
- (UniversalCoder *)copyNoData;

@end

#endif
